#include "radix_sort_serial.h"

int getMax(int arr[], int n) 
{ 
	int mx = arr[0]; 

	for (int i = 1; i < n; i++) 
		if (arr[i] > mx) 
			mx = arr[i]; 
	return mx; 
} 

void countSort(int arr[], int n, int exp) 
{ 
	int* output = (int*)malloc(n * sizeof(int));
	int i, count[10] = {0};

	for (i = 0; i < n; i++) 
		count[ (arr[i]/exp)%10 ]++; 

	for (i = 1; i < 10; i++) 
		count[i] += count[i - 1]; 

	for (i = n - 1; i >= 0; i--) 
	{ 
		output[count[ (arr[i]/exp)%10 ] - 1] = arr[i]; 
		count[ (arr[i]/exp)%10 ]--; 
	} 

	for (i = 0; i < n; i++) 
		arr[i] = output[i];
	free(output);
} 

void radix_sort(int arr[], int n) 
{ 
	int m = getMax(arr, n);

	for (int exp = 1; m/exp > 0; exp *= 10) 
		countSort(arr, n, exp);
} 

void print(int arr[], int n) 
{ 
	for (int i = 0; i < n; i++) 
		printf("%d: %d\n",i, arr[i]);
} 

void rng(int* arr, int n) {
	int seed = 13516013;
	srand(seed);
	for(long i = 0; i < n; i++) {
		arr[i] = (int)rand();
	}
}

int main(int argc, char *argv[]) {
	int N;
	if (argc == 2) {
		N = strtol(argv[1], NULL, 10);
	} else {
		printf("ERROR: ./radix_sort <array_length>\n");
		return 1;
	}
	int arr[N];
	rng(arr,N);
	clock_t begin = clock();
	radix_sort(arr, N);
	clock_t end = clock();
	double time = (double)(end - begin) * 1000000 / CLOCKS_PER_SEC;
	print(arr,N);
	printf("Executed in %lf ms\n",time);
	return 0;
}

